# @summary This defined_type provides way to sync vault's secret content with files in filesystem.
#
# A description of what this defined type does
#
# @example
#   vault_tools::vault_file_sync { 'secret/data/certs':
#     dest => '/tmp',
#     file_specs => {
#       owner => 'nginx',
#       group => 'nginx',
#     },
#     keys_to_place => ["cert.pem","fullchain.pem","privkey.pem"],
#     per_file_specs => {
#       'cert.pem' => {
#         owner => 'root',
#         group => 'root',
#       }
#     },
#     vault_client => {approle_path_segment => 'v1/auth/approle/'},     
#   }
#
# @param vault_path
#  Defines path to get secret from
#
# @param vault_client
#  Hash defines vault connection parametes, check available parameters in (vault-lookup)[https://forge.puppet.com/modules/puppet/vault_lookup/readme] module
#
# @param dest
#  Defines path to place files into
#
# @param keys_to_place
#  Define keys from vault's secret to place in $dest directory
#
# @param per_file_specs
#  Define per file config in format of `file` resource type, e.g.
#  ```
#  file_specs => {
#    '<VAULT_FILE_KEY>'   => {
#      <any param of `file` type> => ... 
#    },
#    '<VAULT_FILE_KEY_2>' => {
#      <any param of `file` type> => ...
#    }
#  }
#  ```
#
# @param file_specs
#  Define linux permissions for each file created by this resource type
#
define vault_tools::vault_file_sync (
  Optional[Array[String]] $keys_to_place   = undef,
  Hash $per_file_specs                     = {},
  Hash $vault_client                       = {},
  String $vault_path                       = $title,
  String $dest                             = undef,
  Hash $file_specs                         = {
    owner => 'root',
    group => 'root',
    mode  => '0644',
  },
) {
  $data = vault_lookup::lookup($vault_path, $vault_client)

  File {
    * => $file_specs,
  }

  $data.unwrap.each |String $key, String $content| {
    if ($keys_to_place == undef or $key in $keys_to_place) {
      file { "${dest}/${key}":
        content => Sensitive($content),
        *       => get($per_file_specs, String($key,'%p'), {}),
      }
    }
  }
}
